# sage_setup: distribution = sagemath-environment
r"""
Feature for testing the presence of PARI/GP.
"""

# *****************************************************************************
#       Copyright (C) 2024 Matthias Koeppe
#
#  Distributed under the terms of the GNU General Public License (GPL)
#  as published by the Free Software Foundation; either version 2 of
#  the License, or (at your option) any later version.
#                  https://www.gnu.org/licenses/
# *****************************************************************************

import subprocess

from . import Executable, Feature, FeatureTestResult


class Gp(Executable):
    r"""
    A :class:`~sage.features.Feature` describing the presence of the ``gp``
    executable.

    EXAMPLES::

        sage: from sage.features.pari import Gp
        sage: Gp().is_present()                                # needs sage.libs.pari
        FeatureTestResult('gp_executable', True)
    """
    def __init__(self):
        r"""
        TESTS::

            sage: from sage.features.pari import Gp
            sage: isinstance(Gp(), Gp)
            True
        """
        Executable.__init__(self, "gp_executable", executable='gp',
                            spkg='pari', type='standard')


class PariFeature(Feature):
    r"""
    A :class:`~sage.features.Feature` describing the presence of a PARI/GP functionality.

    EXAMPLES::

        sage: from sage.features.pari import PariFeature
        sage: F = PariFeature('pari_seadata', 'ellmodulareqn(211)')
        sage: F.is_present()                                   # optional - pari_seadata
        FeatureTestResult('pari_seadata', True)
    """
    def __init__(self, name, command, **kwds):
        Feature.__init__(self, name,
                         spkg=f"pypi/passagemath-{name.replace('_', '-')}",
                         **kwds)
        self.command = command

    def _is_present(self):
        try:
            from sage.libs.pari import pari
        except ImportError as exception:
            return FeatureTestResult(self, False,
                                     reason=f"Interface sage.libs.pari cannot be imported: {exception}")
        try:
            pari(self.command)
        except Exception as exception:
            return FeatureTestResult(self, False,
                                     reason=f"PARI command '{self.command}' failed: {exception}")
        return FeatureTestResult(self, True)


def all_features():
    # Tests according to https://pari.math.u-bordeaux.fr/packages.html
    # and https://pari.math.u-bordeaux.fr/archives/pari-users-2508/msg00041.html
    return [
        PariFeature('pari_elldata', 'ellinit("11a1")'),
        PariFeature('pari_galpol', 'galoisgetname(12,1)'),
        PariFeature('pari_seadata', 'ellmodulareqn(211)'),
        PariFeature('pari_seadata_big', 'ellmodulareqn(521)'),
        PariFeature('pari_seadata_small', 'ellmodulareqn(11)', type='standard'),
        PariFeature('pari_galdata', 'polgalois(x^8-2)', type='standard'),
        PariFeature('pari_nflistdata', 'nflist("A5")'),
        PariFeature('pari_nftables', 'readvec(Str(default(datadir), "/nftables/T77.gp"))'),
    ]



# sage_setup: distribution = sagemath-environment
r"""
Feature for testing if the machine is 32-bit.
"""
from . import Feature, FeatureTestResult
import sys


class Is32Bit(Feature):
    r"""
    A :class:`~sage.features.Feature`.

    EXAMPLES::

        sage: from sage.features.bitness import Is32Bit
        sage: Is32Bit()
        Feature('32_bit')
        sage: Is32Bit() is Is32Bit()
        True
    """
    def __init__(self):
        Feature.__init__(self, '32_bit')

    def _is_present(self):
        r"""
        Test whether the machine is 32-bit.

        EXAMPLES::

            sage: from sage.features.bitness import Is32Bit
            sage: Is32Bit()._is_present()  # random
            FeatureTestResult('32_bit', False)
        """
        return sys.maxsize <= (1 << 32)  # cf. sage.doctest.sources.bitness_value


def all_features():
    return [Is32Bit()]



# sage_setup: distribution = sagemath-environment
r"""
Feature for testing the presence of ``sympow``
"""

# *****************************************************************************
#       Copyright (C) 2024 Matthias Koeppe
#
#  Distributed under the terms of the GNU General Public License (GPL)
#  as published by the Free Software Foundation; either version 2 of
#  the License, or (at your option) any later version.
#                  https://www.gnu.org/licenses/
# *****************************************************************************

import subprocess

from . import Executable, PythonModule
from .join_feature import JoinFeature


class Sympow(Executable):
    r"""
    A :class:`~sage.features.Feature` describing the presence of the ``sympow``
    binary.

    EXAMPLES::

        sage: from sage.features.sympow import Sympow
        sage: Sympow().is_present()                                # needs sympow
        FeatureTestResult('sympow_executable', True)
    """
    def __init__(self):
        r"""
        TESTS::

            sage: from sage.features.sympow import Sympow
            sage: isinstance(Sympow(), Sympow)
            True
        """
        Executable.__init__(self, "sympow_executable", executable='sympow',
                            spkg='sympow', type='standard')


def all_features():
    return [JoinFeature("sympow",
                        (Sympow(),
                         PythonModule('sage.lfunctions.sympow')),
                        spkg='sagemath_sympow', type='standard')]



# sage_setup: distribution = sagemath-environment
r"""
Feature for testing the presence of FROBBY
"""

# *****************************************************************************
#       Copyright (C) 2024 Matthias Koeppe
#
#  Distributed under the terms of the GNU General Public License (GPL)
#  as published by the Free Software Foundation; either version 2 of
#  the License, or (at your option) any later version.
#                  https://www.gnu.org/licenses/
# *****************************************************************************

import subprocess
from . import Executable, FeatureTestResult, PythonModule
from .join_feature import JoinFeature


class Frobby(Executable):
    r"""
    A :class:`~sage.features.Feature` describing the presence of :ref:`frobby <spkg_frobby>`.

    EXAMPLES::

        sage: from sage.features.frobby import Frobby
        sage: Frobby().is_present()  # optional - frobby
        FeatureTestResult('frobby_executable', True)
    """
    def __init__(self):
        r"""
        TESTS::

            sage: from sage.features.frobby import Frobby
            sage: isinstance(Frobby(), Frobby)
            True
        """
        Executable.__init__(self, "frobby_executable", executable='frobby',
                            spkg='frobby')


def all_features():
    return [JoinFeature("frobby",
                        (Frobby(),
                         PythonModule('sage.interfaces.frobby')),
                        spkg='sagemath_frobby')]



# sage_setup: distribution = sagemath-environment
r"""
Features for testing the presence of ``flatter``
"""

from . import Executable


class flatter(Executable):
    """
    A :class:`~sage.features.Feature` describing the presence of ``flatter``.

    EXAMPLES::

        sage: from sage.features.flatter import flatter
        sage: flatter().is_present()  # optional - flatter
        FeatureTestResult('flatter', True)
    """
    def __init__(self):
        r"""
        TESTS::

            sage: from sage.features.flatter import flatter
            sage: isinstance(flatter(), flatter)
            True
        """
        Executable.__init__(self, "flatter", executable="flatter")


def all_features():
    return [flatter()]



# sage_setup: distribution = sagemath-environment
r"""
Features for testing the presence of ``bliss``
"""

# *****************************************************************************
#       Copyright (C) 2016 Julian Rüth
#                     2018 Jeroen Demeyer
#                     2021 Matthias Koeppe
#
#  Distributed under the terms of the GNU General Public License (GPL)
#  as published by the Free Software Foundation; either version 2 of
#  the License, or (at your option) any later version.
#                  https://www.gnu.org/licenses/
# *****************************************************************************

from . import CythonFeature, PythonModule
from .join_feature import JoinFeature


TEST_CODE = """
# distutils: language=c++
# distutils: libraries=bliss

cdef extern from "bliss/graph.hh" namespace "bliss":
    cdef cppclass Graph:
        Graph(const unsigned int)

from cysignals.signals cimport sig_on, sig_off

sig_on()
Graph(1)
sig_off()
"""


class BlissLibrary(CythonFeature):
    r"""
    A :class:`~sage.features.Feature` which describes whether the :ref:`Bliss library <spkg_bliss>` is
    present and functional.

    EXAMPLES::

        sage: from sage.features.bliss import BlissLibrary
        sage: BlissLibrary().require()  # optional - libbliss
    """

    def __init__(self):
        r"""
        TESTS::

            sage: from sage.features.bliss import BlissLibrary
            sage: BlissLibrary()
            Feature('libbliss')
        """
        CythonFeature.__init__(self, "libbliss", test_code=TEST_CODE,
                               spkg='bliss',
                               url='http://www.tcs.hut.fi/Software/bliss/')


class Bliss(JoinFeature):
    r"""
    A :class:`~sage.features.Feature` which describes whether the :mod:`sage.graphs.bliss`
    module is available in this installation of Sage.

    EXAMPLES::

        sage: from sage.features.bliss import Bliss
        sage: Bliss().require()  # optional - bliss
    """
    def __init__(self):
        r"""
        TESTS::

            sage: from sage.features.bliss import Bliss
            sage: Bliss()
            Feature('bliss')
        """
        JoinFeature.__init__(self, "bliss",
                             [PythonModule("sage.graphs.bliss", spkg='sagemath_bliss',
                                           url='http://www.tcs.hut.fi/Software/bliss/')])


def all_features():
    return [Bliss()]
